==> dolfin-0.9.3.tar.gz: dolfin-0.9.3/site-packages/dolfin/compile_extension_module.py <==
"This module provides functionality to compile PyDOLFIN compatible extension modules."

__author__ = "Johan Hake (hake@simula.no)"
__date__ = "2009-08-15 -- 2009-09-18"
__copyright__ = "Copyright (C) 2009 Johan Hake"
__license__  = "GNU LGPL Version 2.1"

import sys
import os
import re
import numpy
import instant
import hashlib
import types



# Import PyDOLFIN
import dolfin

from dolfin.jit import mpi_jit_decorator

__all__ = ["compile_extension_module",
           "expression_to_code_fragments",
           "math_header"]

# A list of supported math builtins
_math_builtins = [
    # cmath functions:
    "cos", "sin", "tan", "acos", "asin", "atan", "atan2",
    "cosh", "sinh", "tanh",
    "exp", "frexp", "ldexp", "log", "log10", "modf",
    "pow", "sqrt", "ceil", "fabs", "floor", "fmod"]

math_header = "\n// cmath functions\n" + \
              "\n".join("using std::%s;"%mf for mf in _math_builtins) + \
              "\nconst double pi = DOLFIN_PI;\n\n"

_cpp_keywords = ["auto","const","double","float","int","short","struct","unsigned",
                 "break","continue","else","for","long","signed","switch","void",
                 "case","default","enum","goto","register","sizeof","typedef",
                 "char","do","extern","if","return","static","union","while",
                 "asm","dynamic_cast","namespace","reinterpret_cast","try",
                 "bool","explicit","new","static_cast","typeid","volatile",
                 "catch","false","operator","template","typename",
                 "class","friend","private","this","using",
                 "const_cast","inline","public","throw","virtual",
                 "delete","mutable","protected","true","wchar_t",
                 "or","and","xor","not"]

def expression_to_code_fragments(expr, defaults, arguments):
    "A help function which extract a dict with code snippets from an expression"

    # Autodetect variables from function strings
    variables = set()

    for i, c in enumerate(expr):
        # Find groups of connected alphanumeric letters
        symbols = re.findall(r"([a-zA-Z_][\w]*)", c)
        assert isinstance(symbols, list)
        variables.update(symbols)
        for sym in symbols:
            if sym in _cpp_keywords:
                raise TypeError, "The c++ keyword '%s' was detected in C++ expression."%sym
        # NOTE: A hack to get around an ambiguous overloading of
        #       dolfin::pow(double,int)
        if "pow" in symbols:
            c = c.replace("pow","std::pow")
        expr[i] = c

    # Remove any variables defined in the arguments list
    variables.difference_update(arguments)

    # Remove the builtin math functions from the variables (and 'pi', we use DOLFIN_PI)
    variables.difference_update(_math_builtins+["pi"])

    # Remove the numerals from the variables
    numerals = [v for v in variables if v[0] in "0123456789"]
    variables.difference_update(numerals)

    # Generate code for member variables
    members_code = "\n".join("  double %s;" % name for name in variables)

    # Generate constructor code for initialization of member variables
    constructor_code = "\n".join("    %s = %s;" %(name,str(defaults.get(name,0.0)))
                                 for name in variables)

    # Connect the code fragments using the function template code
    fragments = {}
    fragments["members"]        = members_code
    fragments["constructor"]    = constructor_code

    # Return the code fragments
    return fragments


def get_dolfin_include_dir():
    # Get system configuration
    (dolfin_include_dirs,
     dummy,
     dummy,
     dummy) = instant.header_and_libs_from_pkgconfig("dolfin")

    # Let swig see the installed dolfin swig files
    dolfin_include_found = False
    for inc_dir in dolfin_include_dirs:
        # Check if dolfin.h is installed in dolfin_include_dir
        if os.path.exists(os.path.join(inc_dir, "dolfin.h")):
            return os.path.abspath(inc_dir)
    raise OSError, "Could not find 'dolfin.h', make sure it is installed in a directory that 'pkg-config --cflags dolfin' returns."

def configure_instant():
    """Check system requirements

    Returns a dict with kwargs that can be passed to instant.build_module.
    """
    instant_kwargs = {}
    # Get system configuration
    (instant_kwargs['include_dirs'],
     flags,
     instant_kwargs['libraries'],
     instant_kwargs['library_dirs']) = instant.header_and_libs_from_pkgconfig("dolfin")

    # Let swig see the installed dolfin swig files
    swig_include_dirs = []
    dolfin_include_found = False
    ufc_include_found = False
    for inc_dir in instant_kwargs['include_dirs']:
        # Check if dolfin is installed in inc_dir
        if os.path.exists(os.path.join(inc_dir, "dolfin", "swig", "dolfin.i")):
            dolfin_include_found = True
            if inc_dir not in swig_include_dirs:
                swig_include_dirs.append(inc_dir)

        # Check if ufc is installed in inc_dir
        if os.path.exists(os.path.join(inc_dir, "swig", "ufc.i")):
            ufc_include_found = True
            if inc_dir not in swig_include_dirs:
                swig_include_dirs.append(inc_dir)

    if not dolfin_include_found:
        raise OSError, """Didn't find dolfin.i in include paths returned from pkg-config.
Please make sure that your DOLFIN installation corresponds with the one returned by pkg-config"""

    if not ufc_include_found:
        raise OSError, """Didn't find ufc.i in include paths returned from pkg-config.
Please make sure that your UFC installation corresponds with the one returned by pkg-config"""

    # Check if UFC is importable and what version of swig was used to
    # create the UFC extension module
    try: import ufc
    except: raise OSError, "Please install the python extenstion module of UFC on your system.\n"

    # Check that the form compiler will use the same swig version
    # that UFC was compiled with
    if not instant.check_swig_version(ufc.__swigversion__,same=True):
        raise OSError, """The python extension module of UFC was not compiled with the present version of swig.
Install swig version %s or recompiled UFC with present swig
"""%ufc.__swigversion__

    # Check that the form compiler will use the same swig version
    # that PyDOLFIN was compiled with
    if not instant.check_swig_version(dolfin.cpp.__swigversion__,same=True):
        raise OSError, """PyDOLFIN was not compiled with the present version of swig.
Install swig version %s or recompiled PyDOLFIN with present swig
"""%dolfin.cpp.__swigversion__

    # Check for boost installation
    # Set a default directory for the boost installation
    if sys.platform == "darwin":
        # use fink as default
        default = os.path.join(os.path.sep,"sw")
    else:
        default = os.path.join(os.path.sep,"usr")

    # If BOOST_DIR is not set use default directory
    boost_dir = os.getenv("BOOST_DIR", default)
    boost_is_found = False
    for inc_dir in ["", "include"]:
        if os.path.isfile(os.path.join(boost_dir, inc_dir, "boost", "version.hpp")):
            boost_include_dir = os.path.join(boost_dir, inc_dir)
            boost_is_found = True
            break

    if not boost_is_found:
        raise OSError, """The Boost library was not found.
If Boost is installed in a nonstandard location,
set the environment variable BOOST_DIR.
"""

    instant_kwargs['swig_include_dirs'] = swig_include_dirs
    instant_kwargs['include_dirs'].append(numpy.get_include())
    instant_kwargs['include_dirs'].append(boost_include_dir)
    instant_kwargs['system_headers'] = ["cmath", "iostream",
                                        "stdexcept","numpy/arrayobject.h",
                                        "dolfin.h","boost/shared_ptr.hpp"]
    instant_kwargs['swigargs'] =['-c++','-I.']

    instant_kwargs['cppargs'] = []

    return instant_kwargs

_pydolfin_shared_ptr_classes = ["Function","SubDomain","Mesh","DofMap","FiniteElement","FunctionSpace"]

_additional_declarations = r"""
%%init%%{
import_array();
%%}

%%include exception.i

%(additional_declarations)s

%%pythoncode %%{
import dolfin.cpp
'''
A hack to get passed a bug in swig.
This is fixed in swig version 1.3.37
%%}
%(dolfin_import_statement)s
%%pythoncode %%{
'''
%%}
%(shared_ptr_declarations)s
"""

def expect_list_of(argtype,arg,argname):
    assert(isinstance(argtype,type))
    error_str = "Provide a 'tuple' or 'list' with '%s', for the '%s' argument"%(argtype.__name__,argname)
    if arg is None:
        return []
    if not isinstance(arg,(types.NoneType,list,tuple)):
        raise TypeError, error_str
    if not all(isinstance(s, str) for s in arg):
        raise TypeError, error_str
    return arg

def expect_arg(argtype,arg,argname):
    if isinstance(argtype,(tuple,list)):
        all(isinstance(s, type) for s in argtype)
    else:
        assert(isinstance(argtype,type))
        type_str = "'%s'"%argtype.__name__

    # Check the type of the argument
    if not isinstance(arg,argtype):
        if isinstance(argtype,(tuple,list)):
            if len(argtype) > 2:
                argtype_str = ", ".join("'%s'"%s.__name__ for s in argtype[:-1]) + " or " + "'%s'"%argtype[-1]
            else:
                argtype_str = " or ".join("'%s'"%s.__name__ for s in argtype)
        else:
            argtype_str = "'%s'"%s.__name__
        raise TypeError, "Expected a '%s' for the '%s' argument"%argname


@mpi_jit_decorator
def compile_extension_module(code, module_name="",
                            recompile=False,
                            dolfin_import_files=None,
                            additional_declarations="",
                            additional_system_headers=None,
                            use_numpy_typemaps=True):
    """ Compile a PyDOLFIN compatible extension module, based on C++ DOLFIN code"""
    # Check the provided arguments
    expect_arg(str,code,"first")
    expect_arg(str,module_name,"module_name")
    expect_arg(str,additional_declarations,"additional_declarations")
    expect_arg(bool,recompile,"recompile")
    expect_arg(bool,use_numpy_typemaps,"use_numpy_typemaps")
    additional_system_headers = \
                expect_list_of(str,additional_system_headers,"additional_system_headers")
    dolfin_import_files = \
                expect_list_of(str,dolfin_import_files,"dolfin_import_files")

    # Check that the code does not use 'using namespace dolfin'
    if re.search("using[ \n]+namespace[ \n]+dolfin",code):
        raise ValueError, "Do not use 'using namespace dolfin'. Include the code in namespace dolfin {...} instead."

    # Check that the code is included in dolfin namespace
    if not re.search("namespace[ \n]+dolfin[ \n]{",code):
        raise ValueError, "Include the C++ code in the 'dolfin namspace', otherwise SWIG cannot figure out the right PyDOLFIN types."

    # Create unique module name for this application run
    if module_name is "":
        module_name = "dolfin_compile_code_%s" % hashlib.md5(repr(code) +\
                                            instant.get_swig_version() +\
                                            dolfin.__version__).hexdigest()
    # Check the handed import files
    if dolfin_import_files:
        dolfin_include_dir = get_dolfin_include_dir()
        for f in dolfin_import_files:
            if not os.path.isfile(os.path.join(dolfin_include_dir,"dolfin",f)):
                raise OSError, "Could not find installed '%s' DOLFIN file."% f

        dolfin_import_files = [os.path.join("dolfin",f) for f in dolfin_import_files]

    # Check cache
    if not recompile:
        compiled_module = instant.import_module(module_name)

        if compiled_module:
            return compiled_module

    sys.stdout.flush()
    dolfin.info("Calling DOLFIN just-in-time (JIT) compiler, this may take some time.")
    # Configure instant and add additional system headers
    instant_kwargs = configure_instant()
    instant_kwargs["system_headers"] += additional_system_headers

    declaration_strs = {"additional_declarations":""}
    if dolfin_import_files is None:
        declaration_strs["dolfin_import_statement"] = r"%import dolfin/swig/dolfin.i"
    else:
        declaration_strs["dolfin_import_statement"] = \
                        "\n".join(['%%import(module="dolfin.cpp") %s'%f
                                   for f in dolfin_import_files])

    # If using numpy typemaps add  %include "numpy.i"
    # FIXME: Need to fix some standard way to apply the types maps...
    if use_numpy_typemaps:
        raise NotImplementedError
        declaration_strs["additional_declarations"] += '\n%include "numpy.i"\n'

    # Add any provided additional declarations
    if additional_declarations is not None:
        declaration_strs["additional_declarations"] += additional_declarations

    # Add any shared_ptr declarations
    shared_ptr_declarations = extract_shared_ptr_declaration(code, _pydolfin_shared_ptr_classes)

    declaration_strs["additional_declarations"] += shared_ptr_declarations[0]
    declaration_strs["shared_ptr_declarations"]  = shared_ptr_declarations[1]

    # Compile extension module with instant
    compiled_module = instant.build_module(\
             code              = code,
             additional_declarations = _additional_declarations%declaration_strs,
             signature         = module_name,
             **instant_kwargs)

    sys.stdout.flush()
    return compiled_module

def extract_shared_ptr_declaration(code, shared_ptr_classes):
    " Extract any declaration for shared_ptr"
    # Check if there are any classes that is derived from any of the
    # shared_ptr classes in PyDOLFIN and declare if any
    shared_ptr_format = "SWIG_SHARED_PTR(%s,dolfin::%s)"
    derived_shared_ptr_format = "SWIG_SHARED_PTR_DERIVED(%s,dolfin::%s,dolfin::%s)"
    derived_and_base = [(derived, base) for derived, base in \
                        re.findall(r"class[ ]+([\w]+)[ ]*: public[ ]+([\w:]+)", code)\
                        if base.split("::")[-1] in shared_ptr_classes]
    
    bases = set([base for derived, base in derived_and_base])
    shared_ptr_declarations = "\n".join([shared_ptr_format%
                                         (base, base) \
                                         for base in bases])
    shared_ptr_declarations += "\n".join([derived_shared_ptr_format%
                                          (derived, base, derived) \
                                          for derived, base in derived_and_base])

    if len(shared_ptr_declarations) > 0:
        return """
//Uncomment these to produce code for std::tr1::shared_ptr
//#define SWIG_SHARED_PTR_NAMESPACE std
//#define SWIG_SHARED_PTR_SUBNAMESPACE tr1
%include <boost_shared_ptr.i>
""","""
// Swig shared_ptr macro declarations
%s
"""%shared_ptr_declarations
    else:
        return ("\n","\n")

==> dolfin-0.9.4.tar.gz: dolfin-0.9.4/site-packages/dolfin/compile_extension_module.py <==
"This module provides functionality to compile PyDOLFIN compatible extension modules."

__author__ = "Johan Hake (hake@simula.no)"
__date__ = "2009-08-15 -- 2009-10-05"
__copyright__ = "Copyright (C) 2009 Johan Hake"
__license__  = "GNU LGPL Version 2.1"

import sys
import os
import re
import numpy
import instant
import hashlib
import types



# Import PyDOLFIN
import dolfin

from dolfin.jit import mpi_jit_decorator

__all__ = ["compile_extension_module",
           "expression_to_code_fragments",
           "math_header"]

# A list of supported math builtins
_math_builtins = [
    # cmath functions:
    "cos", "sin", "tan", "acos", "asin", "atan", "atan2",
    "cosh", "sinh", "tanh",
    "exp", "frexp", "ldexp", "log", "log10", "modf",
    "pow", "sqrt", "ceil", "fabs", "floor", "fmod"]

math_header = "\n// cmath functions\n" + \
              "\n".join("using std::%s;"%mf for mf in _math_builtins) + \
              "\nconst double pi = DOLFIN_PI;\n\n"

_cpp_keywords = ["auto","const","double","float","int","short","struct","unsigned",
                 "break","continue","else","for","long","signed","switch","void",
                 "case","default","enum","goto","register","sizeof","typedef",
                 "char","do","extern","if","return","static","union","while",
                 "asm","dynamic_cast","namespace","reinterpret_cast","try",
                 "bool","explicit","new","static_cast","typeid","volatile",
                 "catch","false","operator","template","typename",
                 "class","friend","private","this","using",
                 "const_cast","inline","public","throw","virtual",
                 "delete","mutable","protected","true","wchar_t",
                 "or","and","xor","not"]

def expression_to_code_fragments(expr, defaults, arguments):
    "A help function which extract a dict with code snippets from an expression"

    expr = list(expr)

    # Autodetect variables from function strings
    variables = set()

    for i, c in enumerate(expr):
        # Find groups of connected alphanumeric letters
        symbols = re.findall(r"([a-zA-Z_][\w]*)", c)
        assert isinstance(symbols, list)
        variables.update(symbols)
        for sym in symbols:
            if sym in _cpp_keywords:
                raise TypeError, "The c++ keyword '%s' was detected in C++ expression."%sym
        # NOTE: A hack to get around an ambiguous overloading of
        #       dolfin::pow(double,int)
        if "pow" in symbols:
            c = c.replace("pow","std::pow")
        expr[i] = c

    # Remove any variables defined in the arguments list
    variables.difference_update(arguments)

    # Remove the builtin math functions from the variables (and 'pi', we use DOLFIN_PI)
    variables.difference_update(_math_builtins+["pi"])

    # Remove the numerals from the variables
    numerals = [v for v in variables if v[0] in "0123456789"]
    variables.difference_update(numerals)

    # Generate code for member variables
    members_code = "\n".join("  double %s;" % name for name in variables)

    # Generate constructor code for initialization of member variables
    constructor_code = "\n".join("    %s = %s;" %(name,str(defaults.get(name,0.0)))
                                 for name in variables)

    # Connect the code fragments using the function template code
    fragments = {}
    fragments["members"]        = members_code
    fragments["constructor"]    = constructor_code

    # Return the code fragments
    return fragments


def get_dolfin_include_dir():
    # Get system configuration
    (dolfin_include_dirs,
     dummy,
     dummy,
     dummy) = instant.header_and_libs_from_pkgconfig("dolfin")

    # Let swig see the installed dolfin swig files
    dolfin_include_found = False
    for inc_dir in dolfin_include_dirs:
        # Check if dolfin.h is installed in dolfin_include_dir
        if os.path.exists(os.path.join(inc_dir, "dolfin.h")):
            return os.path.abspath(inc_dir)
    raise OSError, "Could not find 'dolfin.h', make sure it is installed in a directory that 'pkg-config --cflags dolfin' returns."

def configure_instant():
    """Check system requirements

    Returns a dict with kwargs that can be passed to instant.build_module.
    """
    instant_kwargs = {}
    # Get system configuration
    (instant_kwargs['include_dirs'],
     flags,
     instant_kwargs['libraries'],
     instant_kwargs['library_dirs']) = instant.header_and_libs_from_pkgconfig("dolfin")

    # Let swig see the installed dolfin swig files
    swig_include_dirs = []
    dolfin_include_found = False
    ufc_include_found = False
    for inc_dir in instant_kwargs['include_dirs']:
        # Check if dolfin is installed in inc_dir
        if os.path.exists(os.path.join(inc_dir, "dolfin", "swig", "dolfin.i")):
            dolfin_include_found = True
            if inc_dir not in swig_include_dirs:
                swig_include_dirs.append(inc_dir)

        # Check if ufc is installed in inc_dir
        if os.path.exists(os.path.join(inc_dir, "swig", "ufc.i")):
            ufc_include_found = True
            if inc_dir not in swig_include_dirs:
                swig_include_dirs.append(inc_dir)

    if not dolfin_include_found:
        raise OSError, """Didn't find dolfin.i in include paths returned from pkg-config.
Please make sure that your DOLFIN installation corresponds with the one returned by pkg-config"""

    if not ufc_include_found:
        raise OSError, """Didn't find ufc.i in include paths returned from pkg-config.
Please make sure that your UFC installation corresponds with the one returned by pkg-config"""

    # Check if UFC is importable and what version of swig was used to
    # create the UFC extension module
    try: import ufc
    except: raise OSError, "Please install the python extenstion module of UFC on your system.\n"

    # Check that the form compiler will use the same swig version
    # that UFC was compiled with
    if not instant.check_swig_version(ufc.__swigversion__,same=True):
        raise OSError, """The python extension module of UFC was not compiled with the present version of swig.
Install swig version %s or recompiled UFC with present swig
"""%ufc.__swigversion__

    # Check that the form compiler will use the same swig version
    # that PyDOLFIN was compiled with
    if not instant.check_swig_version(dolfin.cpp.__swigversion__,same=True):
        raise OSError, """PyDOLFIN was not compiled with the present version of swig.
Install swig version %s or recompiled PyDOLFIN with present swig
"""%dolfin.cpp.__swigversion__

    # Check for boost installation
    # Set a default directory for the boost installation
    if sys.platform == "darwin":
        # use fink as default
        default = os.path.join(os.path.sep,"sw")
    else:
        default = os.path.join(os.path.sep,"usr")

    # If BOOST_DIR is not set use default directory
    boost_dir = os.getenv("BOOST_DIR", default)
    boost_is_found = False
    for inc_dir in ["", "include"]:
        if os.path.isfile(os.path.join(boost_dir, inc_dir, "boost", "version.hpp")):
            boost_include_dir = os.path.join(boost_dir, inc_dir)
            boost_is_found = True
            break

    if not boost_is_found:
        raise OSError, """The Boost library was not found.
If Boost is installed in a nonstandard location,
set the environment variable BOOST_DIR.
"""

    instant_kwargs['swig_include_dirs'] = swig_include_dirs
    instant_kwargs['include_dirs'].append(numpy.get_include())
    instant_kwargs['include_dirs'].append(boost_include_dir)
    instant_kwargs['system_headers'] = ["cmath", "iostream",
                                        "stdexcept","numpy/arrayobject.h",
                                        "dolfin.h","boost/shared_ptr.hpp"]
    instant_kwargs['swigargs'] =['-c++','-I.']

    instant_kwargs['cppargs'] = []

    return instant_kwargs

_pydolfin_shared_ptr_classes = ["GenericFunction","Expression","Function",
                                "SubDomain","Mesh","DofMap","FiniteElement",
                                "FunctionSpace"]

_additional_declarations = r"""
%%init%%{
import_array();
%%}

%%include exception.i

%(additional_declarations)s

%%pythoncode %%{
import dolfin.cpp
'''
A hack to get passed a bug in swig.
This is fixed in swig version 1.3.37
%%}
%(dolfin_import_statement)s
%%pythoncode %%{
'''
%%}
%(shared_ptr_declarations)s
"""

def expect_list_of(argtype,arg,argname):
    assert(isinstance(argtype,type))
    error_str = "Provide a 'tuple' or 'list' with '%s', for the '%s' argument"%(argtype.__name__,argname)
    if arg is None:
        return []
    if not isinstance(arg,(types.NoneType,list,tuple)):
        raise TypeError, error_str
    if not all(isinstance(s, str) for s in arg):
        raise TypeError, error_str
    return arg

def expect_arg(argtype,arg,argname):
    if isinstance(argtype,(tuple,list)):
        all(isinstance(s, type) for s in argtype)
    else:
        assert(isinstance(argtype,type))
        type_str = "'%s'"%argtype.__name__

    # Check the type of the argument
    if not isinstance(arg,argtype):
        if isinstance(argtype,(tuple,list)):
            if len(argtype) > 2:
                argtype_str = ", ".join("'%s'"%s.__name__ for s in argtype[:-1]) + " or " + "'%s'"%argtype[-1]
            else:
                argtype_str = " or ".join("'%s'"%s.__name__ for s in argtype)
        else:
            argtype_str = "'%s'"%s.__name__
        raise TypeError, "Expected a '%s' for the '%s' argument"%argname


@mpi_jit_decorator
def compile_extension_module(code, module_name="",
                            recompile=False,
                            dolfin_import_files=None,
                            additional_declarations="",
                            additional_system_headers=None,
                            use_numpy_typemaps=True):
    """ Compile a PyDOLFIN compatible extension module, based on C++ DOLFIN code"""
    # Check the provided arguments
    expect_arg(str,code,"first")
    expect_arg(str,module_name,"module_name")
    expect_arg(str,additional_declarations,"additional_declarations")
    expect_arg(bool,recompile,"recompile")
    expect_arg(bool,use_numpy_typemaps,"use_numpy_typemaps")
    additional_system_headers = \
                expect_list_of(str,additional_system_headers,"additional_system_headers")
    dolfin_import_files = \
                expect_list_of(str,dolfin_import_files,"dolfin_import_files")

    # Check that the code does not use 'using namespace dolfin'
    if re.search("using[ \n]+namespace[ \n]+dolfin",code):
        raise ValueError, "Do not use 'using namespace dolfin'. Include the code in namespace dolfin {...} instead."

    # Check that the code is included in dolfin namespace
    if not re.search("namespace[ \n]+dolfin[ \n]{",code):
        raise ValueError, "Include the C++ code in the 'dolfin namspace', otherwise SWIG cannot figure out the right PyDOLFIN types."

    # Create unique module name for this application run
    if module_name is "":
        module_name = "dolfin_compile_code_%s" % hashlib.md5(repr(code) +\
                                            instant.get_swig_version() +\
                                            dolfin.__version__).hexdigest()
    # Check the handed import files
    if dolfin_import_files:
        dolfin_include_dir = get_dolfin_include_dir()
        for f in dolfin_import_files:
            if not os.path.isfile(os.path.join(dolfin_include_dir,"dolfin",f)):
                raise OSError, "Could not find installed '%s' DOLFIN file."% f

        dolfin_import_files = [os.path.join("dolfin",f) for f in dolfin_import_files]

    # Check cache
    if not recompile:
        compiled_module = instant.import_module(module_name)

        if compiled_module:
            return compiled_module

    sys.stdout.flush()
    dolfin.info("Calling DOLFIN just-in-time (JIT) compiler, this may take some time.")
    # Configure instant and add additional system headers
    instant_kwargs = configure_instant()
    instant_kwargs["system_headers"] += additional_system_headers

    declaration_strs = {"additional_declarations":""}
    if dolfin_import_files:
        using_reduced_import = True
        declaration_strs["dolfin_import_statement"] = \
                        "\n".join(['%%import(module="dolfin.cpp") %s'%f
                                   for f in dolfin_import_files])
    else:
        using_reduced_import = False
        declaration_strs["dolfin_import_statement"] = r"%import dolfin/swig/dolfin.i"

    # If using numpy typemaps add  %include "numpy.i"
    # FIXME: Need to fix some standard way to apply the types maps...
    if use_numpy_typemaps:
        raise NotImplementedError
        declaration_strs["additional_declarations"] += '\n%include "numpy.i"\n'

    # Add any provided additional declarations
    if additional_declarations is not None:
        declaration_strs["additional_declarations"] += additional_declarations

    # Add any shared_ptr declarations
    shared_ptr_declarations = extract_shared_ptr_declaration(code, _pydolfin_shared_ptr_classes, using_reduced_import)

    declaration_strs["additional_declarations"] += shared_ptr_declarations[0]
    declaration_strs["shared_ptr_declarations"]  = shared_ptr_declarations[1]

    # Compile extension module with instant
    compiled_module = instant.build_module(\
             code              = code,
             additional_declarations = _additional_declarations%declaration_strs,
             signature         = module_name,
             **instant_kwargs)

    sys.stdout.flush()
    return compiled_module

def extract_shared_ptr_declaration(code, shared_ptr_classes, using_reduced_import=True):
    " Extract any declaration for shared_ptr"
    # Check if there are any classes that is derived from any of the
    # shared_ptr classes in PyDOLFIN and declare if any
    shared_ptr_format = "SWIG_SHARED_PTR(%s,dolfin::%s)"
    derived_shared_ptr_format = "SWIG_SHARED_PTR_DERIVED(%s,dolfin::%s,dolfin::%s)"
    derived_and_base = [(derived, base) for derived, base in \
                        re.findall(r"class[ ]+([\w]+)[ ]*: public[ ]+([\w:]+)", code)\
                        if base.split("::")[-1] in shared_ptr_classes]
    
    bases = set([base for derived, base in derived_and_base])

    if using_reduced_import:
        for derived_function in ["Expression","Function"]:
            if derived_function in bases:
                bases.remove(derived_function)
                bases.add("GenericFunction")
                derived_and_base.insert(0, (derived_function,"GenericFunction"))

        shared_ptr_declarations = "\n".join([shared_ptr_format%
                                             (base, base) \
                                             for base in bases])
        shared_ptr_declarations += "\n"
    
    else:
        shared_ptr_declarations = ""

    shared_ptr_declarations += "\n".join([derived_shared_ptr_format%
                                          (derived, base, derived) \
                                          for derived, base in derived_and_base])

    if len(shared_ptr_declarations) > 0:
        return """
//Uncomment these to produce code for std::tr1::shared_ptr
//#define SWIG_SHARED_PTR_NAMESPACE std
//#define SWIG_SHARED_PTR_SUBNAMESPACE tr1
%include <boost_shared_ptr.i>
""","""
// Swig shared_ptr macro declarations
%s
"""%shared_ptr_declarations
    else:
        return ("\n","\n")
